#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''****************************************************************************
**
** Copyright (C) 2011 Jedrzej Nowak <pigmej@gmail.com>
**
** This file may be used under the terms of the GNU General Public License
** version 2 as published by the Free Software Foundation and appearing in the
** file GPL.txt included in the packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
****************************************************************************'''


__version__ = 1.46

__max_file_size__ = 1024.0

__server_address__ = 'https://wklej.to/api/'

import xmlrpclib
import sys
from optparse import OptionParser, OptionGroup
import os
import base64


def get_uid(source):
    """
    Pobiera uid paste z url itp (http://wklej.to/uid, wklej.to/uid...)
    """
    try:
        uid = source[source.rindex('/') + 1:]
    except ValueError:
        uid = source
    return uid


def get_force(force):
    return 0 if force else __version__

def create(srv, create, key, passw, force):
    """
    Tworzy paste
    """
    content = None
    if create: #... -c /path/to/file
        f_size = os.path.getsize(opt.create) / 1024.0
        if f_size > __max_file_size__:
            sys.stderr.write(u"Plik jest zbyt duży (%0.1f KB), limit to %0.1f KB" % (f_size, __max_file_size__))
            sys.exit(1)
        with open(opt.create, 'r') as f:
            content = f.read()
    else:
        content = sys.stdin.read()
    return srv.create_paste(opt.type, content, passw, opt.create or "", key, get_force(opt.force))

def get(srv, get, key, force):
    """
    Pobiera paste
    """
    type = opt.type or 'text'
    return srv.get_paste(get_uid(opt.get), type, opt.key, get_force(opt.force))

def remove(srv, remove, passw, force):
    """
    Usuwa paste
    """
    return srv.remove_paste(get_uid(opt.remove), opt.password, get_force(opt.force))

def types(srv):
    """
    Pobiera typy
    """
    return srv.types()

def ver(srv):
    """
    Pobiera wersje serwera
    """
    res = srv.server_ver()
    return res[0], str(res[1])

def result(res):
    """
    Parsuje wyjscie, pisze na wyjscie, zamyka skrypt
    """
    if res[0] == 1:
        sys.stdout.write(res[1].encode('utf-8'))
        sys.stdout.write("\n")
        sys.exit(0)
    else:
        sys.stderr.write(res[1].encode('utf-8'))
        sys.stderr.write("\n")
        sys.exit(1)

def get_srv():
    return xmlrpclib.ServerProxy(__server_address__)

if __name__ == '__main__':
    usage = "%prog [options]\nhttp://wklej.to/page/klienty"
    parser = OptionParser(usage)

    actions = OptionGroup(parser, u"Opcje określające akcje")

    actions.add_option('-c',
        '--create',
        metavar = "FILE_PATH",
        help = u'Użyj aby utworzyć nową wklejkę')

    actions.add_option('-g',
        '--get',
        metavar = 'UID',
        help = u"Użyj aby pobrać paste o podanym uid.")

    actions.add_option('-r',
        '--remove',
        metavar = 'UID',
        help = u"Użyj aby usunąć paste o podanym uid.")

    actions.add_option('-t',
        '--type',
        default = 'text',
        help = u"Użyj aby określić typ wklejki.")

    helpers = OptionGroup(parser, u"Opcje pomocnicze")

    helpers.add_option('--types',
        action = 'store_true',
        help = u"Umożliwia wylistowanie typów kolorowania dostępnych po stronie serwera.")

    helpers.add_option('-v',
        '--ver',
        action = 'store_true',
        help = u"Umożliwia pobranie wersji serwera")

    helpers.add_option("--force",
        action = 'store_true',
        help = u"Wymusza wykonanie akcji.")

    safety = OptionGroup(parser, u"Opcje związane z bezpieczeństwem")

    safety.add_option('-p',
        '--password',
        default = '',
        help = u"Hasło do usuwania wklejki")

    safety.add_option("-k",
        '--key',
        default = '',
        help = u'klucz który służy do zaszyfrowania zawartości.')

    parser.add_option_group(actions)
    parser.add_option_group(helpers)
    parser.add_option_group(safety)

    (opt, args) = parser.parse_args()

    srv = get_srv()

    if opt.create:
        result(create(srv, opt.create, opt.key, opt.password, opt.force))
    elif opt.get:
        result(get(srv, opt.get, opt.key, opt.force))
    elif opt.remove:
        result(remove(srv, opt.remove, opt.password))
    elif opt.types:
        result(types(srv))
    elif opt.ver:
        result(ver(srv))
    else:
        result(create(srv, opt.create, opt.key, opt.password, opt.force))

